from collections import MutableMapping, Container
from datetime import datetime, timedelta


class LimitedTimeTable(MutableMapping, Container):

    def __init__(self, time_span):
        self.__storage = dict()
        self.__time_span = None
        self.time_span = time_span

    @property
    def time_span(self):
        return self.__time_span

    @time_span.setter
    def time_span(self, value):
        if isinstance(value, timedelta):
            self.__time_span = value
        else:
            raise TypeError('Invalid argument\'s value type! Expected for "datetime.timedelta".')

    @property
    def oldest(self):
        value = None
        if self.__len__() > 0:
            value = min(self.__storage.keys())
        return value

    @property
    def newest(self):
        value = None
        if self.__len__() > 0:
            value = max(self.__storage.keys())
        return value

    def oldest_keys(self, size):
        for key in self.__get_slice(0, size):
            yield key

    def oldest_values(self, size):
        for key in self.oldest_keys(size):
            yield self.__storage.get(key)

    def oldest_items(self, size):
        for key in self.oldest_keys(size):
            yield (key, self.__storage.get(key))

    def newest_keys(self, size):
        for key in self.__get_slice(-size, None):
            yield key

    def newest_values(self, size):
        for key in self.newest_keys(size):
            yield self.__storage.get(key)

    def newest_items(self, size):
        for key in self.newest_keys(size):
            yield (key, self.__storage.get(key))

    def __get_slice(self, start, end):
        keys = sorted(self.keys())
        return keys[start:end]

    def __getitem__(self, item):
        return self.__storage.__getitem__(item)

    def __setitem__(self, key, value):
        if not isinstance(key, datetime):
            raise TypeError('Invalid key type! Expected for "datetime.datetime".')
        now = datetime.now()
        if key > now:
            raise ValueError('Can\'t set item from future!')
        oldest = self.oldest
        if (oldest is not None) and (oldest != key):
            longest_time_span = now - oldest
            if longest_time_span >= self.time_span:  # Item is too old for current timetable
                self.__delitem__(oldest)
        return self.__storage.__setitem__(key, value)

    def __delitem__(self, key):
        return self.__storage.__delitem__(key)

    def __len__(self):
        return self.__storage.__len__()

    def __iter__(self):
        return self.__storage.__iter__()

    def __contains__(self, item):
        return self.__storage.__contains__(item)


import psutil
import platform
import os
import subprocess
import re
from collections import Callable
from datetime import datetime, timedelta
from pyspectator.monitoring import AbcMonitor
from pyspectator.collection import LimitedTimeTable


class Processor(AbcMonitor):

    # region initialization

    def __init__(self, monitoring_latency, stats_interval=None):
        super().__init__(monitoring_latency)
        self.__name = Processor.__get_processor_name()
        self.__count = psutil.cpu_count()
        self.__load = None
        self.__temperature = None
        # Init temperature reader
        self.__temperature_reader = Processor.__get_processor_temperature_reader()
        # Prepare to collect statistics
        if stats_interval is None:
            stats_interval = timedelta(hours=1)
        self.__load_stats = LimitedTimeTable(stats_interval)
        self.__temperature_stats = LimitedTimeTable(stats_interval)
        # Read updating value at first time
        self._monitoring_action()

    # endregion

    # region properties

    @property
    def name(self):
        return self.__name

    @property
    def count(self):
        return self.__count

    @property
    def load(self):
        return self.__load

    @property
    def temperature(self):
        return self.__temperature

    @property
    def load_stats(self):
        return self.__load_stats

    @property
    def temperature_stats(self):
        return self.__temperature_stats

    # endregion

    # region methods

    def _monitoring_action(self):
        now = datetime.now()
        self.__load = psutil.cpu_percent()
        self.__load_stats[now] = self.__load
        if isinstance(self.__temperature_reader, Callable):
            self.__temperature = self.__temperature_reader()
            self.__temperature_stats[now] = self.__temperature

    @classmethod
    def __get_processor_name(cls):
        cpu_name = None
        os_name = platform.system()
        if os_name == 'Windows':
            cpu_name = platform.processor()
        elif os_name == 'Darwin':
            os.environ['PATH'] = os.environ['PATH'] + os.pathsep + '/usr/sbin'
            command = 'sysctl -n machdep.cpu.brand_string'
            cpu_name = subprocess.check_output(command).strip()
        elif os_name == 'Linux':
            all_info = subprocess.check_output('cat /proc/cpuinfo', shell=True).strip()
            for line in all_info.split(os.linesep.encode()):
                line = line.decode()
                if 'model name' in line:
                    cpu_name = re.sub('.*model name.*:', str(), line, 1).strip()
                    break
        return cpu_name

    @classmethod
    def __get_processor_temperature_reader(cls):
        reader = None
        os_name = platform.system()
        if os_name == 'Windows':
            reader = cls.__windows_processor_temperature_reader()
        elif os_name == 'Darwin':
            pass  # TODO: try to use C lib  - https://github.com/lavoiesl/osx-cpu-temp
        elif os_name == 'Linux':
            reader = cls.__linux__processor_temperature_reader()
        return reader

    # endregion

    @classmethod
    def __windows_processor_temperature_reader(cls):
        import wmi
        import pythoncom
        def temperature_reader():
            pythoncom.CoInitialize()
            w = wmi.WMI(namespace='root\\wmi')
            temperature = w.MSAcpi_ThermalZoneTemperature()[0].CurrentTemperature
            temperature = int(temperature / 10.0 - 273.15)
            return temperature
        return temperature_reader

    @classmethod
    def __linux__processor_temperature_reader(cls):
        reader = None

        def temperature_reader1():
            temperature = open('/sys/devices/LNXSYSTM:00/LNXTHERM:00/LNXTHERM:01/thermal_zone/temp').read().strip()
            temperature = int(temperature) // 1000
            return temperature

        def temperature_reader2():
            temperature = open('/sys/bus/acpi/devices/LNXTHERM:00/thermal_zone/temp').read().strip()
            temperature = int(temperature) // 1000
            return temperature

        def temperature_reader3():
            temperature = open('/proc/acpi/thermal_zone/THM0/temperature').read().strip()
            temperature = temperature.lstrip('temperature :').rstrip(' C')
            return int(temperature)

        def temperature_reader4():
            temperature = open('/proc/acpi/thermal_zone/THRM/temperature').read().strip()
            temperature = temperature.lstrip('temperature :').rstrip(' C')
            return int(temperature)

        def temperature_reader5():
            temperature = open('/proc/acpi/thermal_zone/THR1/temperature').read().strip()
            temperature = temperature.lstrip('temperature :').rstrip(' C')
            return int(temperature)

        if os.path.exists('/sys/devices/LNXSYSTM:00/LNXTHERM:00/LNXTHERM:01/thermal_zone/temp') is True:
            reader = temperature_reader1
        elif os.path.exists('/sys/bus/acpi/devices/LNXTHERM:00/thermal_zone/temp') is True:
            reader = temperature_reader2
        elif os.path.exists('/proc/acpi/thermal_zone/THM0/temperature') is True:
            reader = temperature_reader3
        elif os.path.exists('/proc/acpi/thermal_zone/THRM/temperature') is True:
            reader = temperature_reader4
        elif os.path.exists('/proc/acpi/thermal_zone/THR1/temperature') is True:
            reader = temperature_reader5
        return reader

    pass


import subprocess
import platform
from os import linesep
from time import sleep
from pyspectator.convert import UnitByte


clear_command = "cls" if platform.system() == "Windows" else "clear"


def clear():
    subprocess.call(clear_command, shell=True)


def print_hr(space_before=False, space_after=False):
    before = linesep if space_before else ''
    after = linesep if space_after else ''
    print(before + '-' * 80 + after)


class Format(object):

    @staticmethod
    def temperature(value):
        formatted_value = ''
        if isinstance(value, (int, float)):
            formatted_value = str(value) + '°C'
        return formatted_value

    @staticmethod
    def byte_value(value):
        formatted_value = ''
        if isinstance(value, (int, float)):
            value, unit = UnitByte.auto_convert(value)
            value = '{0:.2f}'.format(value)
            unit = UnitByte.get_name_reduction(unit)
            formatted_value = value + unit
        return formatted_value

    @staticmethod
    def percent(value):
        formatted_value = ''
        if isinstance(value, (int, float)):
            formatted_value = str(value) + '%'
        return formatted_value


def main(computer):
    print('Start monitoring system...')
    print_hr(space_after=True)
    # Show system info for ~16 seconds
    for _ in range(16):
        clear()
        # Display general information about computer
        print('OS: ' + str(computer.os))
        print('Boot time: {0}; Uptime: {1}'.format(computer.boot_time, computer.uptime))
        print('')
        # Display CPU info
        print('CPU name: ' + str(computer.processor.name))
        print('Amount of CPU cores: ' + str(computer.processor.count))
        print('CPU load: ' + Format.percent(computer.processor.load))
        cpu_temperature = 'unknown'
        if computer.processor.temperature is not None:
            cpu_temperature = Format.temperature(computer.processor.temperature)
        print('CPU temperature: ' + cpu_temperature)
        print('')
        # Display network info
        print('Hostname: ' + str(computer.hostname))
        print('Network interface: ' + str(computer.network_interface.name))
        print('MAC address: ' + str(computer.network_interface.hardware_address))
        print('IP: {0}; Mask: {2}; Gateway: {3}'.format(
            computer.network_interface.ip_address,
            computer.network_interface.broadcast_address,
            computer.network_interface.subnet_mask,
            computer.network_interface.default_route
        ))
        print('Sent data: {0}; Received data: {1}'.format(
            Format.byte_value(computer.network_interface.bytes_sent),
            Format.byte_value(computer.network_interface.bytes_recv)
        ))
        print('')
        # Display virtual memory info
        print('Virtual memory: use {0} from {1}, {2}'.format(
            Format.byte_value(computer.virtual_memory.available),
            Format.byte_value(computer.virtual_memory.total),
            Format.percent(computer.virtual_memory.used_percent)
        ))
        print('')
        # Display nonvolatile memory info
        output_format1 = '{0:_^16}{1:_^16}{2:_^16}{3:_^16}{4:_^16}'
        output_format2 = '{0: ^16}{1: ^16}{2: ^16}{3: ^16}{4: ^16}'
        print(output_format1.format('Device', 'Total', 'Use', 'Type', 'Mount'))
        for dev in computer.nonvolatile_memory:
            output_text = output_format2.format(
                dev.device,
                Format.byte_value(dev.total),
                Format.percent(dev.used_percent),
                dev.fstype,
                dev.mountpoint
            )
            print(output_text)
        sleep(1)
    print_hr(space_before=True)
    print('Shutdown monitoring system...')


if __name__ == '__main__':
    # Initialize computer instance
    from pyspectator.computer import Computer
    curr_computer = Computer()
    # Start monitoring system
    with curr_computer:
        # Start console interface
        main(curr_computer)


import psutil
from abc import ABCMeta, abstractmethod
from datetime import timedelta, datetime
from pyspectator.monitoring import AbcMonitor
from pyspectator.collection import LimitedTimeTable


class AbsMemory(AbcMonitor, metaclass=ABCMeta):

    # region initialization

    def __init__(self, monitoring_latency, stats_interval=None):
        # Prepare to collect statistic
        if stats_interval is None:
            stats_interval = timedelta(hours=1)
        self.__available_stats = LimitedTimeTable(stats_interval)
        self.__used_percent_stats = LimitedTimeTable(stats_interval)
        # Get info about total and available memory
        try:
            self.__total = self._get_total_memory()
            self.__available = self._get_available_memory()
            now = datetime.now()
            self.__available_stats[now] = self.available
            self.__used_percent_stats[now] = self.used_percent
        except:
            self.__total = None
            self.__available = None
            self.__available_stats.clear()
            self.__used_percent_stats.clear()
        # Init base class
        super().__init__(monitoring_latency)

    # endregion

    # region properties

    @property
    def total(self):
        return self.__total

    @property
    def available(self):
        return self.__available

    @property
    def used(self):
        used_mem = None
        if (self.total is not None) and (self.available is not None):
            used_mem = self.total - self.available
        return used_mem

    @property
    def used_percent(self):
        percent = None
        used = self.used
        if used is not None:
            percent = int(used / self.total * 100)
        return percent

    @property
    def available_stats(self):
        return self.__available_stats

    @property
    def used_percent_stats(self):
        return self.__used_percent_stats

    # endregion

    # region methods

    def _monitoring_action(self):
        try:
            self.__available = self._get_available_memory()
            now = datetime.now()
            self.__available_stats[now] = self.available
            self.__used_percent_stats[now] = self.used_percent
        except:
            self.__available = None
            self.__available_stats.clear()
            self.__used_percent_stats.clear()
            self.stop_monitoring()

    @abstractmethod
    def _get_total_memory(self):
        raise NotImplementedError('Method not implemented by derived class!')

    @abstractmethod
    def _get_available_memory(self):
        raise NotImplementedError('Method not implemented by derived class!')

    # endregion

    pass


class VirtualMemory(AbsMemory):

    def __init__(self, monitoring_latency):
        super().__init__(monitoring_latency)

    def _get_available_memory(self):
        return psutil.virtual_memory().available

    def _get_total_memory(self):
        return psutil.virtual_memory().total


class SwapMemory(AbsMemory):

    def __init__(self, monitoring_latency):
        super().__init__(monitoring_latency)

    def _get_available_memory(self):
        return psutil.swap_memory().free

    def _get_total_memory(self):
        return psutil.swap_memory().total


class NonvolatileMemory(AbsMemory):

    # region initialization

    def __init__(self, monitoring_latency, device):
        dev_info = None
        for current_dev_info in psutil.disk_partitions():
            if current_dev_info.device == device:
                dev_info = current_dev_info
                break
        if dev_info is None:
            raise DeviceNotFoundException('Device {0} not found!'.format(device))
        self.__is_alive = True
        self.__device = device
        self.__mountpoint = dev_info.mountpoint
        self.__fstype = dev_info.fstype
        super().__init__(monitoring_latency)

    # endregion

    # region properties

    @property
    def device(self):
        return self.__device

    @property
    def mountpoint(self):
        return self.__mountpoint

    @property
    def fstype(self):
        return self.__fstype

    @property
    def is_alive(self):
        return self.__is_alive

    # endregion

    # region methods

    def _get_available_memory(self):
        return psutil.disk_usage(self.mountpoint).free

    def _get_total_memory(self):
        return psutil.disk_usage(self.mountpoint).total

    def _monitoring_action(self):
        self.__is_alive = self.device not in NonvolatileMemory.names_connected_devices()
        if self.is_alive:
            super()._monitoring_action()

    @staticmethod
    def instances_connected_devices(monitoring_latency):
        devices = list()
        for current_dev_info in psutil.disk_partitions():
            current_dev = NonvolatileMemory(monitoring_latency, current_dev_info.device)
            devices.append(current_dev)
        return devices

    @staticmethod
    def names_connected_devices():
        return [dev_info.device for dev_info in psutil.disk_partitions()]

    # endregion

    pass


class DeviceNotFoundException(Exception):
    pass



import psutil
import platform
from datetime import datetime
from pyspectator.monitoring import AbcMonitor
from pyspectator.memory import NonvolatileMemory, VirtualMemory, SwapMemory
from pyspectator.processor import Processor
from pyspectator.network import NetworkInterface


class Computer(AbcMonitor):

    # region initialization

    def __init__(self):
        self.datetime_format = '%H:%M:%S %d/%m/%Y'
        self.__raw_boot_time = psutil.boot_time()
        self.__boot_time = datetime.fromtimestamp(self.raw_boot_time).strftime(self.datetime_format)
        self.__hostname = platform.node()
        self.__os = Computer.__get_os_name()
        self.__architecture = platform.machine()
        self.__python_version = '{0} ver. {1}'.format(
            platform.python_implementation(), platform.python_version()
        )
        self.__processor = Processor(monitoring_latency=1)
        self.__nonvolatile_memory = NonvolatileMemory.instances_connected_devices(monitoring_latency=10)
        self.__nonvolatile_memory_devices = set([dev_info.device for dev_info in self.__nonvolatile_memory])
        self.__virtual_memory = VirtualMemory(monitoring_latency=1)
        self.__swap_memory = SwapMemory(monitoring_latency=1)
        self.__network_interface = NetworkInterface(monitoring_latency=3)
        super().__init__(monitoring_latency=3)

    # endregion

    # region properties

    @property
    def processor(self):
        return self.__processor

    @property
    def raw_boot_time(self):
        return self.__raw_boot_time

    @property
    def boot_time(self):
        return self.__boot_time

    @property
    def raw_uptime(self):
        return datetime.now() - datetime.fromtimestamp(self.raw_boot_time)

    @property
    def uptime(self):
        return str(self.raw_uptime).split('.')[0]

    @property
    def os(self):
        return self.__os

    @property
    def architecture(self):
        return self.__architecture

    @property
    def python_version(self):
        return self.__python_version

    @property
    def hostname(self):
        return self.__hostname

    @property
    def nonvolatile_memory(self):
        return self.__nonvolatile_memory

    @property
    def virtual_memory(self):
        return self.__virtual_memory

    @property
    def swap_memory(self):
        return self.__swap_memory

    @property
    def network_interface(self):
        return self.__network_interface

    # endregion

    # region methods

    @classmethod
    def __get_os_name(cls):
        system = '{0} {1}'.format(platform.system(), platform.release()).strip()
        if ('Linux' in system) and ('' not in platform.linux_distribution()):
            system = ' '.join(platform.linux_distribution())
        return system

    def _monitoring_action(self):
        # Look for connected & ejected nonvolatile memory devices
        for dev in self.nonvolatile_memory:
            if not dev.is_alive:
                dev.stop_monitoring()
                self.__nonvolatile_memory_devices.remove(dev.device)
                self.__nonvolatile_memory.remove(dev)
        connected_dev = set(NonvolatileMemory.names_connected_devices()) - self.__nonvolatile_memory_devices
        for dev_name in connected_dev:
            dev = NonvolatileMemory(monitoring_latency=10, device=dev_name)
            self.__nonvolatile_memory.append(dev)
            self.__nonvolatile_memory_devices.add(dev_name)

    def start_monitoring(self, all_components=True):
        super().start_monitoring()
        if all_components:
            self.processor.start_monitoring()
            for mem in self.nonvolatile_memory:
                mem.start_monitoring()
            self.virtual_memory.start_monitoring()
            self.network_interface.start_monitoring()

    def stop_monitoring(self, all_components=True):
        if all_components:
            self.processor.stop_monitoring()
            for mem in self.nonvolatile_memory:
                mem.stop_monitoring()
            self.virtual_memory.stop_monitoring()
            self.network_interface.stop_monitoring()
        super().stop_monitoring()

    def __enter__(self):
        self.start_monitoring()

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.stop_monitoring()

    # endregion

    pass



from abc import ABCMeta, abstractmethod
from threading import Timer


class AbcMonitor(metaclass=ABCMeta):
    """Base class for entities, which require repeating event.

    Attributes:
        monitoring (bool): indicator activity of monitor.
        monitoring_latency (int, float): frequency of execution monitor's action.
    """

    # region initialization

    def __init__(self, monitoring_latency):
        self.__monitoring_latency = None
        self.monitoring_latency = monitoring_latency
        self.__monitoring = False

    # endregion

    # region properties

    @property
    def monitoring(self):
        return self.__monitoring

    @property
    def monitoring_latency(self):
        return self.__monitoring_latency

    @monitoring_latency.setter
    def monitoring_latency(self, value):
        self.__monitoring_latency = value

    # endregion

    # region methods

    def start_monitoring(self):
        """Enable periodically monitoring.
        """
        if self.__monitoring is False:
            self.__monitoring = True
            self.__monitoring_action()

    def stop_monitoring(self):
        """Disable periodically monitoring.
        """
        self.__monitoring = False

    def __enter__(self):
        self.start_monitoring()

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.stop_monitoring()

    def __monitoring_action(self):
        if self.__monitoring is True:
            self._monitoring_action()
            Timer(self.monitoring_latency, self.__monitoring_action).start()

    @abstractmethod
    def _monitoring_action(self):
        """Action, which repeated, when monitoring is enabled.
        """
        raise NotImplementedError('Method not implemented by derived class!')

    # endregion

    pass


import sys
import platform
import os

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


def main():
    # Check python version
    if sys.version_info < (3, 0, 0):
        sys.stderr.write('You need python 3.0 or later to run this script!' + os.linesep)
        exit(1)
    # Generate requires
    requires = [
        "psutil >= 2.1.1",
        "netifaces >= 0.10.4",
    ]
    if sys.version_info < (3, 4, 1):
        requires.append("enum34 >= 1.0")
    if platform.system() == 'Windows':
        requires.append('wmi >= 1.4.9')
    # Describe installer
    setup(
        name='pyspectator',
        version='1.0.8',
        author='Maxim Grischuk, Vova Sirenko',
        author_email='uzumaxy@gmail.com',
        maintainer='Maxim Grischuk',
        maintainer_email='uzumaxy@gmail.com',
        packages=['pyspectator'],
        url='https://github.com/opium999/pyspectator',
        download_url='https://github.com/opium999/pyspectator/releases',
        license='BSD',
        description='pyspectator is a cross-platform tool for retrieving full information about computer.',
        long_description=open('README.rst').read(),
        install_requires=requires,
        keywords=[
            'pyspectator', 'spectator',
            'monitoring', 'tool',
            'statistic', 'stats',
            'computer', 'pc', 'server',
            'mem', 'memory',
            'network', 'net', 'io',
            'processor', 'cpu',
            'hdd', 'hard', 'disk', 'drive'
        ],
        platforms='Platform Independent',
        package_data={
            'pyspectator': ['LICENSE', 'README.rst']
        },
        scripts=['console.py'],
        classifiers=[
            'Development Status :: 5 - Production/Stable',
            'Environment :: Console',
            'Environment :: MacOS X',
            'Environment :: Win32 (MS Windows)',
            'Intended Audience :: Developers',
            'Intended Audience :: Information Technology',
            'Intended Audience :: System Administrators',
            'License :: OSI Approved :: BSD License',
            'Natural Language :: English',
            'Operating System :: MacOS :: MacOS X',
            'Operating System :: Microsoft :: Windows :: Windows 7',
            'Operating System :: Microsoft :: Windows :: Windows NT/2000',
            'Operating System :: Microsoft :: Windows :: Windows Server 2003',
            'Operating System :: Microsoft :: Windows :: Windows Server 2008',
            'Operating System :: Microsoft :: Windows :: Windows Vista',
            'Operating System :: Microsoft :: Windows :: Windows XP',
            'Operating System :: Microsoft',
            'Operating System :: OS Independent',
            'Operating System :: POSIX :: BSD :: FreeBSD',
            'Operating System :: POSIX :: Linux',
            'Operating System :: POSIX :: SunOS/Solaris',
            'Operating System :: POSIX',
            'Programming Language :: C',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.0',
            'Programming Language :: Python :: 3.1',
            'Programming Language :: Python :: 3.2',
            'Programming Language :: Python :: 3.3',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: Implementation :: CPython',
            'Programming Language :: Python',
            'Topic :: Software Development :: Libraries :: Python Modules',
            'Topic :: Software Development :: Libraries',
            'Topic :: System :: Benchmark',
            'Topic :: System :: Hardware',
            'Topic :: System :: Monitoring',
            'Topic :: System :: Networking :: Monitoring',
            'Topic :: System :: Networking',
            'Topic :: System :: Systems Administration',
            'Topic :: Utilities',
        ],
    )


if __name__ == '__main__':
    main()


import socket
import psutil
import netifaces as nif
from datetime import timedelta, datetime
from pyspectator.monitoring import AbcMonitor
from pyspectator.collection import LimitedTimeTable


class NetworkInterface(AbcMonitor):

    # region initialization

    def __init__(self,  monitoring_latency, stats_interval=None, ip_address=None):
        super().__init__(monitoring_latency)
        self.__name = None
        self.__hardware_address = None
        if ip_address is None:
            ip_address = NetworkInterface.__get_active_ip_address()
        self.__ip_address = ip_address
        self.__broadcast_address = None
        self.__subnet_mask = None
        self.__default_route = None
        self.__bytes_sent = 0
        self.__bytes_recv = 0
        # Get interface name, network mask and broadcast address
        if self.__ip_address is not None:
            for interface in nif.interfaces():
                addresses = nif.ifaddresses(interface)
                try:
                    af_inet = addresses[nif.AF_INET][0]
                    if af_inet['addr'] != self.__ip_address:
                        continue
                    af_link = addresses[nif.AF_LINK][0]
                    self.__name = NetworkInterface.__check_interface_name(interface)
                    self.__hardware_address = af_link['addr']
                    self.__broadcast_address = af_inet['broadcast']
                    self.__subnet_mask = af_inet['netmask']
                    break
                except (IndexError, KeyError):  # ignore interfaces that don't have MAC or IP
                    continue
        # Get gateway address
        if self.name is not None:
            for gateway_info in nif.gateways()[nif.AF_INET]:
                if self.name in gateway_info:
                    self.__default_route = gateway_info[0]
                    break
        # Prepare to collect statistics
        if stats_interval is None:
            stats_interval = timedelta(hours=1)
        self.__bytes_sent_stats = LimitedTimeTable(stats_interval)
        self.__bytes_recv_stats = LimitedTimeTable(stats_interval)
        # Read updating values at first time
        self._monitoring_action()
        pass

    # endregion

    # region properties

    @property
    def name(self):
        return self.__name

    @property
    def hardware_address(self):
        return self.__hardware_address

    @property
    def ip_address(self):
        return self.__ip_address

    @property
    def broadcast_address(self):
        return self.__broadcast_address

    @property
    def subnet_mask(self):
        return self.__subnet_mask

    @property
    def default_route(self):
        return self.__default_route

    @property
    def bytes_sent(self):
        return self.__bytes_sent

    @property
    def bytes_recv(self):
        return self.__bytes_recv

    @property
    def bytes_sent_stats(self):
        return self.__bytes_sent_stats

    @property
    def bytes_recv_stats(self):
        return self.__bytes_recv_stats

    # endregion

    # region methods

    @classmethod
    def __check_interface_name(cls, name):
        net_io = psutil.net_io_counters(pernic=True)
        if name in net_io:
            return name
        for curr_nif_name in net_io:
            if name in curr_nif_name:
                name = curr_nif_name
                break
        return name

    @classmethod
    def __get_active_ip_address(cls):
        ip_address = None
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        try:
            s.connect(('8.8.8.8', 80))
            ip_address = s.getsockname()[0]
        except:
            s.close()
        return ip_address

    def _monitoring_action(self):
        net_io = psutil.net_io_counters(pernic=True)
        if self.name in net_io:
            net_io = net_io[self.name]
            now = datetime.now()
            self.__bytes_sent = net_io.bytes_sent
            self.__bytes_recv_stats[now] = self.bytes_sent
            self.__bytes_recv = net_io.bytes_recv
            self.__bytes_recv_stats[now] = self.bytes_recv

    # endregion

    pass


from enum import IntEnum


class UnitByte(IntEnum):

    byte = 2 ** 0
    kilobyte = 2 ** 10
    megabyte = 2 ** 20
    gigabyte = 2 ** 30
    terabyte = 2 ** 40
    petabyte = 2 ** 50
    exabyte = 2 ** 60
    zettabyte = 2 ** 70
    yottabyte = 2 ** 80

    @staticmethod
    def convert(byte_value, unitbyte):
        converted_value = None
        if unitbyte in UnitByte:
            converted_value = byte_value / unitbyte.value
        return converted_value

    @staticmethod
    def auto_convert(byte_value):
        used_unitbyte = UnitByte.yottabyte
        converted_value = byte_value / UnitByte.yottabyte
        for unitbyte in UnitByte:
            temp_value = byte_value / unitbyte.value
            if temp_value < 1024:
                used_unitbyte = unitbyte
                converted_value = temp_value
                break
        return converted_value, used_unitbyte

    @staticmethod
    def get_name_reduction(unitbyte):
        reductions_en = {
            UnitByte.byte: 'B',
            UnitByte.kilobyte: 'KB',
            UnitByte.megabyte: 'MB',
            UnitByte.gigabyte: 'GB',
            UnitByte.terabyte: 'TB',
            UnitByte.petabyte: 'PB',
            UnitByte.exabyte: 'EB',
            UnitByte.zettabyte: 'ZB',
            UnitByte.yottabyte: 'YB'
        }
        reduction = None
        if unitbyte in UnitByte:
            reduction = reductions_en[unitbyte]
        return reduction
